@@ -44,6 +44,7 @@ FILE_OPTIONS = {
                             'dogpile.cache.bmemcached',
                             'dogpile.cache.dbm',
                             'dogpile.cache.redis',
+                            'dogpile.cache.redis_sentinel',
                             'dogpile.cache.memory',
                             'dogpile.cache.memory_pickle',
                             'dogpile.cache.null'],
@@ -80,7 +81,7 @@ FILE_OPTIONS = {
                          'If ``dogpile.cache.memcached`` or '
                          '``oslo_cache.memcache_pool`` is used and a given '
                          'host refer to an IPv6 or a given domain refer to '
-                         'IPv6 then you should prefix the given address with'
+                         'IPv6 then you should prefix the given address with '
                          'the address family (``inet6``) '
                          '(e.g ``inet6[::1]:11211``, '
                          '``inet6:[fd12:3456:789a:1::1]:11211``, '
@@ -122,16 +123,39 @@ FILE_OPTIONS = {
                     help='Enable the SASL(Simple Authentication and Security'
                          'Layer) if the SASL_enable is true, else disable.'),
         cfg.StrOpt('memcache_username',
-                   default='',
                    help='the user name for the memcached which SASL enabled'),
         cfg.StrOpt('memcache_password',
-                   default='',
                    secret=True,
                    help='the password for the memcached which SASL enabled'),
+        cfg.StrOpt('redis_server',
+                   default='localhost:6379',
+                   help='Redis server in the format of "host:port"'),
+        cfg.StrOpt('redis_username',
+                   help='the user name for redis'),
+        cfg.StrOpt('redis_password',
+                   secret=True,
+                   help='the password for redis'),
+        cfg.ListOpt('redis_sentinels',
+                    default=['localhost:26379'],
+                    help='Redis sentinel servers in the format of '
+                         '"host:port"'),
+        cfg.FloatOpt('redis_socket_timeout',
+                     default=1.0,
+                     help='Timeout in seconds for every call to a server.'
+                     ' (dogpile.cache.redis and dogpile.cache.redis_sentinel '
+                     'backends only).'),
+        cfg.StrOpt('redis_sentinel_service_name',
+                   default='mymaster',
+                   help='Service name of the redis sentinel cluster.'),
         cfg.BoolOpt('tls_enabled',
                     default=False,
-                    help='Global toggle for TLS usage when comunicating with'
-                    ' the caching servers.'),
+                    help='Global toggle for TLS usage when communicating with'
+                    ' the caching servers. Currently supported by '
+                    '``dogpile.cache.bmemcache``, '
+                    '``dogpile.cache.pymemcache``, '
+                    '``oslo_cache.memcache_pool``, '
+                    '``dogpile.cache.redis`` and '
+                    '``dogpile.cache.redis_sentinel``.'),
         cfg.StrOpt('tls_cafile',
                    default=None,
                    help='Path to a file of concatenated CA certificates in PEM'
@@ -157,7 +181,10 @@ FILE_OPTIONS = {
                    help='Set the available ciphers for sockets created with'
                    ' the TLS context. It should be a string in the OpenSSL'
                    ' cipher list format. If not specified, all OpenSSL enabled'
-                   ' ciphers will be available.'),
+                   ' ciphers will be available. Currently supported by '
+                   '``dogpile.cache.bmemcache``, '
+                   '``dogpile.cache.pymemcache`` and '
+                   '``oslo_cache.memcache_pool``.'),
         cfg.BoolOpt(
             'enable_socket_keepalive',
             default=False,
@@ -217,6 +244,18 @@ FILE_OPTIONS = {
             default=60,
             help='Time in seconds before attempting to add a node '
             'back in the pool in the HashClient\'s internal mechanisms.'),
+        cfg.BoolOpt('enforce_fips_mode',
+                    default=False,
+                    help='Global toggle for enforcing the OpenSSL FIPS mode. '
+                    'This feature requires Python support. '
+                    'This is available in Python 3.9 in all '
+                    'environments and may have been backported to older '
+                    'Python versions on select environments. If the Python '
+                    'executable used does not support OpenSSL FIPS mode, '
+                    'an exception will be raised. Currently supported by '
+                    '``dogpile.cache.bmemcache``, '
+                    '``dogpile.cache.pymemcache`` and '
+                    '``oslo_cache.memcache_pool``.'),
     ],
 }
 
