@@ -21,10 +21,8 @@ from oslo_cache import core
 from oslo_log import log
 from oslo_utils import importutils
 from oslo_utils import timeutils
-import six
 
 from oslo_cache._i18n import _
-from oslo_cache._i18n import _LW
 from oslo_cache import exception
 
 
@@ -417,15 +415,15 @@ class MongoApi(object):
         Refer to MongoDB documentation around TTL index for further details.
         """
         indexes = collection.index_information()
-        for indx_name, index_data in six.iteritems(indexes):
+        for indx_name, index_data in indexes.items():
             if all(k in index_data for k in ('key', 'expireAfterSeconds')):
                 existing_value = index_data['expireAfterSeconds']
                 fld_present = 'doc_date' in index_data['key'][0]
                 if fld_present and existing_value > -1 and ttl_seconds < 1:
-                    msg = _LW('TTL index already exists on db collection '
-                              '<%(c_name)s>, remove index <%(indx_name)s> '
-                              'first to make updated mongo_ttl_seconds value '
-                              'to be  effective')
+                    msg = ('TTL index already exists on db collection '
+                           '<%(c_name)s>, remove index <%(indx_name)s> '
+                           'first to make updated mongo_ttl_seconds value '
+                           'to be  effective')
                     LOG.warning(msg, {'c_name': coll_name,
                                       'indx_name': indx_name})
 
@@ -440,7 +438,7 @@ class MongoApi(object):
 
     def get_multi(self, keys):
         db_results = self._get_results_as_dict(keys)
-        return {doc['_id']: doc['value'] for doc in six.itervalues(db_results)}
+        return {doc['_id']: doc['value'] for doc in db_results.values()}
 
     def _get_results_as_dict(self, keys):
         criteria = {'_id': {'$in': keys}}
@@ -496,8 +494,7 @@ class MongoApi(object):
                                            **self.meth_kwargs)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class AbstractManipulator(object):
+class AbstractManipulator(object, metaclass=abc.ABCMeta):
     """Abstract class with methods which need to be implemented for custom
     manipulation.
 
