@@ -34,7 +34,9 @@ The library has special public value for nonexistent or expired keys called
     from oslo_cache import core
     NO_VALUE = core.NO_VALUE
 """
+import re
 import ssl
+import urllib.parse
 
 import dogpile.cache
 from dogpile.cache import api
@@ -100,6 +102,18 @@ class _DebugProxy(proxy.ProxyBackend):
         self.proxied.delete_multi(keys)
 
 
+def _parse_sentinel(sentinel):
+    # IPv6 (eg. [::1]:6379 )
+    match = re.search(r'^\[(\S+)\]:(\d+)$', sentinel)
+    if match:
+        return (match[1], int(match[2]))
+    # IPv4 or hostname (eg. 127.0.0.1:6379 or localhost:6379)
+    match = re.search(r'^(\S+):(\d+)$', sentinel)
+    if match:
+        return (match[1], int(match[2]))
+    raise exception.ConfigurationError('Malformed sentinel server format')
+
+
 def _build_cache_config(conf):
     """Build the cache region dictionary configuration.
 
@@ -136,57 +150,207 @@ def _build_cache_config(conf):
         conf_dict[arg_key] = argvalue
 
         _LOG.debug('Oslo Cache Config: %s', conf_dict)
-    # NOTE(yorik-sar): these arguments will be used for memcache-related
-    # backends. Use setdefault for url to support old-style setting through
-    # backend_argument=url:127.0.0.1:11211
-    #
-    # NOTE(morgan): If requested by config, 'flush_on_reconnect' will be set
-    # for pooled connections. This can ensure that stale data is never
-    # consumed from a server that pops in/out due to a network partition
-    # or disconnect.
-    #
-    # See the help from python-memcached:
-    #
-    # param flush_on_reconnect: optional flag which prevents a
-    #        scenario that can cause stale data to be read: If there's more
-    #        than one memcached server and the connection to one is
-    #        interrupted, keys that mapped to that server will get
-    #        reassigned to another. If the first server comes back, those
-    #        keys will map to it again. If it still has its data, get()s
-    #        can read stale data that was overwritten on another
-    #        server. This flag is off by default for backwards
-    #        compatibility.
-    #
-    # The normal non-pooled clients connect explicitly on each use and
-    # does not need the explicit flush_on_reconnect
-    conf_dict.setdefault('%s.arguments.url' % prefix,
-                         conf.cache.memcache_servers)
-    for arg in ('dead_retry', 'socket_timeout', 'pool_maxsize',
-                'pool_unused_timeout', 'pool_connection_get_timeout',
-                'pool_flush_on_reconnect'):
-        value = getattr(conf.cache, 'memcache_' + arg)
-        conf_dict['%s.arguments.%s' % (prefix, arg)] = value
+
+    if conf.cache.backend == 'dogpile.cache.redis':
+        if conf.cache.redis_password is None:
+            netloc = conf.cache.redis_server
+        else:
+            if conf.cache.redis_username:
+                netloc = '%s:%s@%s' % (conf.cache.redis_username,
+                                       conf.cache.redis_password,
+                                       conf.cache.redis_server)
+            else:
+                netloc = ':%s@%s' % (conf.cache.redis_password,
+                                     conf.cache.redis_server)
+
+        parts = urllib.parse.ParseResult(
+            scheme=('rediss' if conf.cache.tls_enabled else 'redis'),
+            netloc=netloc, path='', params='', query='', fragment='')
+
+        conf_dict.setdefault(
+            '%s.arguments.url' % prefix,
+            urllib.parse.urlunparse(parts)
+        )
+        for arg in ('socket_timeout',):
+            value = getattr(conf.cache, 'redis_' + arg)
+            conf_dict['%s.arguments.%s' % (prefix, arg)] = value
+    elif conf.cache.backend == 'dogpile.cache.redis_sentinel':
+        for arg in ('password', 'socket_timeout'):
+            value = getattr(conf.cache, 'redis_' + arg)
+            conf_dict['%s.arguments.%s' % (prefix, arg)] = value
+        if conf.cache.redis_username:
+            # TODO(tkajinam): Update dogpile.cache to add username argument,
+            # similarly to password.
+            conf_dict['%s.arguments.connection_kwargs' % prefix] = \
+                {'username': conf.cache.redis_username}
+            conf_dict['%s.arguments.sentinel_kwargs' % prefix] = \
+                {'username': conf.cache.redis_username}
+        conf_dict['%s.arguments.service_name' % prefix] = \
+            conf.cache.redis_sentinel_service_name
+        if conf.cache.redis_sentinels:
+            conf_dict['%s.arguments.sentinels' % prefix] = [
+                _parse_sentinel(s) for s in conf.cache.redis_sentinels]
+    else:
+        # NOTE(yorik-sar): these arguments will be used for memcache-related
+        # backends. Use setdefault for url to support old-style setting through
+        # backend_argument=url:127.0.0.1:11211
+        #
+        # NOTE(morgan): If requested by config, 'flush_on_reconnect' will be
+        # set for pooled connections. This can ensure that stale data is never
+        # consumed from a server that pops in/out due to a network partition
+        # or disconnect.
+        #
+        # See the help from python-memcached:
+        #
+        # param flush_on_reconnect: optional flag which prevents a
+        #        scenario that can cause stale data to be read: If there's more
+        #        than one memcached server and the connection to one is
+        #        interrupted, keys that mapped to that server will get
+        #        reassigned to another. If the first server comes back, those
+        #        keys will map to it again. If it still has its data, get()s
+        #        can read stale data that was overwritten on another
+        #        server. This flag is off by default for backwards
+        #        compatibility.
+        #
+        # The normal non-pooled clients connect explicitly on each use and
+        # does not need the explicit flush_on_reconnect
+        conf_dict.setdefault('%s.arguments.url' % prefix,
+                             conf.cache.memcache_servers)
+
+        for arg in ('dead_retry', 'socket_timeout', 'pool_maxsize',
+                    'pool_unused_timeout', 'pool_connection_get_timeout',
+                    'pool_flush_on_reconnect', 'sasl_enabled', 'username',
+                    'password'):
+            value = getattr(conf.cache, 'memcache_' + arg)
+            conf_dict['%s.arguments.%s' % (prefix, arg)] = value
 
     if conf.cache.tls_enabled:
-        _LOG.debug('Oslo Cache TLS - CA: %s', conf.cache.tls_cafile)
-        tls_context = ssl.create_default_context(cafile=conf.cache.tls_cafile)
+        if conf.cache.backend in ('dogpile.cache.bmemcache',
+                                  'dogpile.cache.pymemcache',
+                                  'oslo_cache.memcache_pool'):
+            _LOG.debug('Oslo Cache TLS - CA: %s', conf.cache.tls_cafile)
+            tls_context = ssl.create_default_context(
+                cafile=conf.cache.tls_cafile)
 
-        if conf.cache.tls_certfile is not None:
-            _LOG.debug('Oslo Cache TLS - cert: %s', conf.cache.tls_certfile)
-            _LOG.debug('Oslo Cache TLS - key: %s', conf.cache.tls_keyfile)
-            tls_context.load_cert_chain(
-                conf.cache.tls_certfile,
-                conf.cache.tls_keyfile,
+            if conf.cache.enforce_fips_mode:
+                if hasattr(ssl, 'FIPS_mode'):
+                    _LOG.info("Enforcing the use of the OpenSSL FIPS mode")
+                    ssl.FIPS_mode_set(1)
+                else:
+                    raise exception.ConfigurationError(
+                        "OpenSSL FIPS mode is not supported by your Python "
+                        "version. You must either change the Python "
+                        "executable used to a version with FIPS mode support "
+                        "or disable FIPS mode by setting "
+                        "the '[cache] enforce_fips_mode' configuration option "
+                        "to 'False'.")
+
+            if conf.cache.tls_certfile is not None:
+                _LOG.debug('Oslo Cache TLS - cert: %s',
+                           conf.cache.tls_certfile)
+                _LOG.debug('Oslo Cache TLS - key: %s', conf.cache.tls_keyfile)
+                tls_context.load_cert_chain(
+                    conf.cache.tls_certfile,
+                    conf.cache.tls_keyfile,
+                )
+
+            if conf.cache.tls_allowed_ciphers is not None:
+                _LOG.debug(
+                    'Oslo Cache TLS - ciphers: %s',
+                    conf.cache.tls_allowed_ciphers,
+                )
+                tls_context.set_ciphers(conf.cache.tls_allowed_ciphers)
+
+            conf_dict['%s.arguments.tls_context' % prefix] = tls_context
+        elif conf.cache.backend in ('dogpile.cache.redis',
+                                    'dogpile.cache.redis_sentinel'):
+            if conf.cache.tls_allowed_ciphers is not None:
+                raise exception.ConfigurationError(
+                    "Limiting allowed ciphers is not supported by "
+                    "the %s backend" % conf.cache.backend)
+            if conf.cache.enforce_fips_mode:
+                raise exception.ConfigurationError(
+                    "FIPS mode is not supported by the %s backend" %
+                    conf.cache.backend)
+
+            conn_kwargs = {}
+            if conf.cache.tls_cafile is not None:
+                _LOG.debug('Oslo Cache TLS - CA: %s', conf.cache.tls_cafile)
+                conn_kwargs['ssl_ca_certs'] = conf.cache.tls_cafile
+            if conf.cache.tls_certfile is not None:
+                _LOG.debug('Oslo Cache TLS - cert: %s',
+                           conf.cache.tls_certfile)
+                _LOG.debug('Oslo Cache TLS - key: %s', conf.cache.tls_keyfile)
+                conn_kwargs.update({
+                    'ssl_certfile': conf.cache.tls_certfile,
+                    'ssl_keyfile': conf.cache.tls_keyfile
+                })
+            if conf.cache.backend == 'dogpile.cache.redis_sentinel':
+                conn_kwargs.update({'ssl': True})
+                conf_dict.setdefault(
+                    '%s.arguments.connection_kwargs' % prefix,
+                    {}).update(conn_kwargs)
+                conf_dict.setdefault(
+                    '%s.arguments.sentinel_kwargs' % prefix,
+                    {}).update(conn_kwargs)
+            else:
+                conf_dict.setdefault(
+                    '%s.arguments.connection_kwargs' % prefix,
+                    {}).update(conn_kwargs)
+        else:
+            raise exception.ConfigurationError(
+                "TLS setting via [cache] tls_enabled is not supported by the "
+                "%s backend. Set [cache] tls_enabled=False or use a different "
+                "backend." % conf.cache.backend
             )
 
-        if conf.cache.tls_allowed_ciphers is not None:
-            _LOG.debug(
-                'Oslo Cache TLS - ciphers: %s',
-                conf.cache.tls_allowed_ciphers,
+    # NOTE(hberaud): Pymemcache support socket keepalive, If it is enable in
+    # our config then configure it to enable this feature.
+    # The socket keepalive feature means that pymemcache will be able to check
+    # your connected socket and determine whether the connection is still up
+    # and running or if it has broken.
+    # This could be used by users who want to handle fine grained failures.
+    if conf.cache.enable_socket_keepalive:
+        if conf.cache.backend != 'dogpile.cache.pymemcache':
+            msg = _(
+                "Socket keepalive is only supported by the "
+                "'dogpile.cache.pymemcache' backend."
             )
-            tls_context.set_ciphers(conf.cache.tls_allowed_ciphers)
+            raise exception.ConfigurationError(msg)
+        import pymemcache
+        socket_keepalive = pymemcache.KeepaliveOpts(
+            idle=conf.cache.socket_keepalive_idle,
+            intvl=conf.cache.socket_keepalive_interval,
+            cnt=conf.cache.socket_keepalive_count)
+        # As with the TLS context above, the config dict below will be
+        # consumed by dogpile.cache that will be used as a proxy between
+        # oslo.cache and pymemcache.
+        conf_dict['%s.arguments.socket_keepalive' % prefix] = socket_keepalive
 
-        conf_dict['%s.arguments.tls_context' % prefix] = tls_context
+    # NOTE(hberaud): The pymemcache library comes with retry mechanisms that
+    # can be used to wrap all kind of pymemcache clients. The retry wrapper
+    # allow you to define how many attempts to make and how long to wait
+    # between attempts. The section below will pass our config
+    # to dogpile.cache to setup the pymemcache retry client wrapper.
+    if conf.cache.enable_retry_client:
+        if conf.cache.backend != 'dogpile.cache.pymemcache':
+            msg = _(
+                "Retry client is only supported by the "
+                "'dogpile.cache.pymemcache' backend."
+            )
+            raise exception.ConfigurationError(msg)
+        import pymemcache
+        conf_dict['%s.arguments.enable_retry_client' % prefix] = True
+        conf_dict['%s.arguments.retry_attempts' % prefix] = \
+            conf.cache.retry_attempts
+        conf_dict['%s.arguments.retry_delay' % prefix] = \
+            conf.cache.retry_delay
+        conf_dict['%s.arguments.hashclient_retry_attempts' % prefix] = \
+            conf.cache.hashclient_retry_attempts
+        conf_dict['%s.arguments.hashclient_retry_delay' % prefix] = \
+            conf.cache.hashclient_retry_delay
+        conf_dict['%s.arguments.dead_timeout' % prefix] = \
+            conf.cache.dead_timeout
 
     return conf_dict
 
